// (C) Copyright 2014-2016, Jaime Ferreira, David Martins de Matos, Ricardo Ribeiro
// Spoken Language Systems Lab, INESC ID, IST/Universidade de Lisboa

// This file is part of XOKDE++.

// XOKDE++ is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.

// XOKDE++ is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA

#ifndef SRC_INCLUDE_CLASSIFIER_H_
#define SRC_INCLUDE_CLASSIFIER_H_

#include "kde/basic_types.h"
#include "dataset/split_dataset.h"
#include "classifier/prediction.h"

namespace xokdepp {

  template<typename label_type>
  class classifier {
  public:
    typedef prediction<label_type> prediction_type;

  private:
    double _correctly_classified = 0;
    double _incorrectly_classified = 0;
    double _evaluated_samples = 0;

  public:
    //TODO this should not be public...
    std::vector<prediction_type> _predictions;

  public:

    void print_evaluation_and_predictions() {
      std::cout << "groundtruth_label | predicted_label     groundth_num | pred_num    correctly_classified" << std::endl;
      evaluate();
      std::cout << std::endl << "final_statistics:" << std::endl;
      std::cout << "Total samples: ";
      std::cout << _correctly_classified + _incorrectly_classified << std::endl;
      std::cout << "Correctly classified: ";
      std::cout << _correctly_classified << " (" << _correctly_classified / _evaluated_samples * 100 << " %)" << std::endl;
      std::cout << "Incorrectly classified: ";
      std::cout << _incorrectly_classified << " (" << _incorrectly_classified / _evaluated_samples * 100 << " %)" << std::endl;
    }

    void evaluate() {
      for (prediction_type prediction : _predictions) {
        if (prediction.correctly_classified()) {
          _correctly_classified++;
        } else {
          _incorrectly_classified++;
        }
        _evaluated_samples++;
      }
    }

    std::vector<prediction_type> const & predictions() const{
    	return _predictions;
    }

    void add_prediction(const label_type &truth) {
      _predictions.push_back(prediction_type(truth));
    }

    void add_prediction(const label_type &truth, const label_type &predicted, double confidence) {
      _predictions.push_back(prediction_type(truth, predicted, confidence));
    }

    double correctly_classified() const {
      return _correctly_classified;
    }

    double incorrectly_classified() const {
      return _incorrectly_classified;
    }

    double evaluated_samples() const {
      return _evaluated_samples;
    }

  };

}

#endif /* SRC_INCLUDE_CLASSIFIER_H_ */
