// (C) Copyright 2014-2016, Jaime Ferreira, David Martins de Matos, Ricardo Ribeiro
// Spoken Language Systems Lab, INESC ID, IST/Universidade de Lisboa

// This file is part of XOKDE++.

// XOKDE++ is free software; you can redistribute it and/or modify it under
// the terms of the GNU General Public License as published by the Free
// Software Foundation; either version 2 of the License, or (at your
// option) any later version.

// XOKDE++ is distributed in the hope that it will be useful, but WITHOUT
// ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
// FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
// for more details.

// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
// USA

#ifndef SRC_INCLUDE_PREDICTION_H_
#define SRC_INCLUDE_PREDICTION_H_

#include <string>
#include <iostream>
#include "kde/basic_types.h"

namespace xokdepp {

  template<typename label_type>
  class prediction {

    label_type _groundtruth_label;

    label_type _predicted_label;

    double _confidence;

    //TODO is it worth it?
//	label_type _second_choice_label;
//	double _second_choice_confidence;

    bool _correctly_classified;

  public:

    //DAVID: FIXME: should not init labels this way
    prediction() :
        _groundtruth_label(label_type("unknown", -1)), _predicted_label(label_type("unknown", -1)), _confidence(0), _correctly_classified(
            false) {
    }

    prediction(label_type groundtruth_label) :
        _groundtruth_label(groundtruth_label), _predicted_label(label_type("unknown", -1)), _confidence(0), _correctly_classified(
            false) {
    }

    prediction(label_type groundtruth_label, label_type predicted_label, double confidence) :
        _groundtruth_label(groundtruth_label), _predicted_label(predicted_label), _confidence(confidence), _correctly_classified(
            _groundtruth_label == _predicted_label) {
    }

    bool correctly_classified() const {
      return _correctly_classified;
    }

    data_type likelihood(){
    	return _confidence;
    }

    label_type groundtruth_label(){
    	return _groundtruth_label;
    }

    friend std::ostream &operator<<(std::ostream &o, const prediction &prediction) {
      o << std::get<0>(prediction._groundtruth_label) << " | " << std::get<0>(prediction._predicted_label) << "    ";
      o << std::get<1>(prediction._groundtruth_label) << " | " << std::get<1>(prediction._predicted_label) << "    ";
      o << prediction._correctly_classified;
      return o;
    }

  };

}

#endif /* SRC_INCLUDE_PREDICTION_H_ */
